from django.urls import path
from . import views
app_name = 'nets_core_auth_api'

urlpatterns = [
    path('login/', views.auth_login, name='login'),
    path('logout/', views.auth_logout, name='logout'),
    path('authenticate/', views.auth, name='authenticate'),
    path('update/', views.update_user, name='update'),
    path('getProfile/', views.auth_get_profile, name='getProfile'),
    path('requestDelete/', views.request_delete_user_account, name='requestDelete'),
    path('delete/', views.delete_user_account, name='delete'),
]


# Generated by Django 4.1.7 on 2023-08-16 05:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('nets_core', '0006_verificationcode_device'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verificationcode',
            name='device',
            field=models.ForeignKey(blank=True, max_length=150, null=True, on_delete=django.db.models.deletion.CASCADE, to='nets_core.userdevice'),
        ),
    ]



# Generated by Django 5.0.4 on 2024-05-05 19:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nets_core', '0010_permission_role_userrole_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='role',
            name='enabled',
            field=models.BooleanField(default=True, verbose_name='Enabled?'),
        ),
    ]



from channels.auth import AuthMiddlewareStack, UserLazyObject
from channels.db import database_sync_to_async
from channels.middleware import BaseMiddleware
from django.contrib.auth.models import AnonymousUser
from oauth2_provider.models import AccessToken




@database_sync_to_async
def get_user(scope):
    """
    Return the user model instance associated with the given scope.
    If no user is retrieved, return an instance of `AnonymousUser`.
    """
    headers = dict(scope['headers'])
    user = None
    if b'authorization' in headers:

        access_token = headers[b'authorization'].decode()
        access_token = access_token.replace('Bearer ', '')

        try:
            token = AccessToken.objects.get(token=access_token)

            if token.is_valid():

                user = token.user
                # return user
            else:
                "received invalid token, delete from database"

                token.delete()
        except AccessToken.DoesNotExist:
            "token does not exist.... should be block future requests with invalid tokens?"
            pass
    u = user or AnonymousUser()

    return u


class AuthTokenMiddleware(BaseMiddleware):
    """
    Token authorization middleware for Django Channels 
    And OAUTH2 PROVIDER
    """
    
    def __init__(self, app):
        self.app = app

    def populate_scope(self, scope):
        if "user" not in scope:
            scope["user"] = UserLazyObject()

    async def resolve_scope(self, scope):
        if "user" not in scope or scope['user'] == AnonymousUser():
            scope["user"]._wrapped = await get_user(scope)

    async def __call__(self, scope, receive, send):
        scope = dict(scope)
        # Scope injection/mutation per this middleware's needs.
        self.populate_scope(scope)
        # Grab the finalized/resolved scope
        await self.resolve_scope(scope)

        return await self.app(scope, receive, send)
        # return await super().__call__(scope, receive, send)


def AuthTokenMiddlewareStack(inner): return (
    AuthMiddlewareStack(AuthTokenMiddleware(inner)))



import calendar
import time
import uuid
import re
from datetime import date, datetime

from django.contrib.contenttypes.models import ContentType
from django.apps import apps
from django.utils import timezone

import pytz
from django.conf import settings
from django.utils.dateparse import parse_datetime


def local_datetime(s: str, tz: str = settings.TIME_ZONE) -> datetime:
    naive = parse_datetime(s)
    if not naive:
        raise ValueError("local_datetime: Not a valid datetime")

    return pytz.timezone(tz).localize(naive, is_dst=None)


def get_client_ip(request):
    META_PRECEDENCE_ORDER = (
        "HTTP_X_FORWARDED_FOR",
        "X_FORWARDED_FOR",  # <client>, <proxy1>, <proxy2>
        "HTTP_CLIENT_IP",
        "HTTP_X_REAL_IP",
        "HTTP_X_FORWARDED",
        "HTTP_X_CLUSTER_CLIENT_IP",
        "HTTP_FORWARDED_FOR",
        "HTTP_FORWARDED",
        "HTTP_VIA",
        "REMOTE_ADDR",
    )

    for h in META_PRECEDENCE_ORDER:
        ip = request.META.get(h, None)
        if ip:
            if "," in ip:
                ip = ip.split(",")[0]
            return ip
    return None


def generate_int_uuid(size=None):
    u = uuid.uuid1()
    n_random = "{}".format(u.time_low)

    time_epoch = str(calendar.timegm(time.gmtime()))

    u_id = "{}{}".format(n_random, time_epoch)

    if size:
        u_id = u_id[:size]
    return int(u_id)


def get_upload_path(instance, filename):
    folder = instance._meta.model_name
    path = ""
    if instance and hasattr(instance, "project"):
        path = f"PSMDOC_PROJ_{instance.project.id}/"

    path += "{}".format(folder) if folder.endswith("/") else "{}/".format(folder)

    today = timezone.now()
    date_path = today.strftime("%Y/%m/%d/")
    path = "{}{}".format(path, date_path)
    path = "{}{}".format(path, filename)
    return path


def check_perm(user, action, project=None):
    from nets_core.models import Permission, RolePermission

    project_content_type = None
    project_id = None
    if user.is_superuser:
        return True

    if project:
        project_content_type = ContentType.objects.get_for_model(project)
        project_id = project.id

    if not Permission.objects.filter(codename=action).exists():
        # create permission and return False because this permission does not exist
        permission = Permission.objects.create(
            codename=action, name=action.replace("_", " ").capitalize()
        )
        return False

    if project:

        try:
            project_member_model = apps.get_model(
                settings.NETS_CORE_PROJECT_MEMBER_MODEL
            )
            try:
                member = project_member_model.objects.get(user=user, project=project)
                if hasattr(member, "enabled") and not member.enabled:
                    return False
                if hasattr(member, "is_superuser") and member.is_superuser:
                    return True
                
                if hasattr(member, 'role'):
                    if action.startswith('role:'):
                        return member.role.name.lower() == action.split(':')[1].lower()
                
            except project_member_model.DoesNotExist:
                return False
        except:
            raise Exception(
                "check_perm failed NETS_CORE_PROJECT_MEMBER_MODEL not set in settings"
            )

        user_roles = member.user.roles.filter(
            project_content_type=project_content_type, project_id=project_id
        )
        
        
        if user_roles.exists():            
            roles = [u.role for u in user_roles]
            return RolePermission.objects.filter(
                role__in=roles,
                permission__codename=action.lower(),
            ).exists()
        else:
            return False

    else:
        user_roles = user.roles.filter(role__enabled=True)
        user_perms = []
        for r in user_roles:
            user_perms += r.role.permissions.all()
        print(f"User: {user}", user_roles, user_perms)
        for p in user_perms:
            print(p.codename)
            if p.codename == action:
                return True

        return False



import os
from pathlib import Path
from django.db import connections

def create_nets_core_functions(using: str = "default"):
    # functions are store in nets_core/procedures/functions/*.sql
    # each file state with nets_core_[db_engine]_[function_name].sql
    # db_engine can be postgresql, mysql, sqlite, etc
    # Example: nets_core_postgresql_model_to_json.sql will create a function to convert a model to json in postgresql
    # we need to check db engine and create the function accordingly
    
    # use stdout to print the sql query
    
    import logging
    logger = logging.getLogger(__name__)
    
    
    connection = connections[using]
    engine = connection.vendor
    logger.warning(f"Creating nets_core functions for {using} database engine {engine}")
    function_files = os.path.join(Path(__file__).parent, "functions")
    for file in os.scandir(function_files):
        if file.is_file():
            if file.name.startswith(f"nets_core_{engine}"):
                with open(file.path, "r") as f:
                    query = f.read()
                    logger.warning(f"Executing {file.name} function")
                    with connection.cursor() as cursor:
                        cursor.execute(query)
    connection.close()
    return True

        


import hmac
import hashlib
from django.apps import apps
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.contrib.auth.hashers import check_password, make_password
from base64 import b64decode, b64encode
from oauthlib import common
from oauth2_provider.models import Application, AccessToken, RefreshToken


from nets_core.models import Permission, Role, UserRole, VerificationCode


# TODO: create middleware to restring token_access with device_uuid


def validate_verification_code(user, code: str):
    vcode = VerificationCode.objects.filter(user=user).last()
    if not vcode:
        return False
    return vcode.validate(code)


def authenticate(
    user, code: str, client_id: str, client_secret: str, device_uuid: str = None
) -> dict:
    """
    Check client_id and client_secret, validate verification code and
    create access and refresh tokens for oauth2
    raise Exception if any of this fail.

    Parameters:
    user (instance): Instance of settings.AUTH_MODEL_MODEL

    Returns:
    dict: {"access_token": str, "refresh_token": str, "token_expires": datetime }

    Raise:
    Exception: If client_id, client_secret or code are not valid.

    example of usage:
        try:
            tokens = authenticate(user, '123456', 'client_id_app', 'client_secret_app')
            # return tokens to your user here

        except Exception as e:
            msg = e.__str__()
            # your code if fail here
    """
    try:
        oauth_app = Application.objects.get(client_id=client_id)
        if not oauth_app.client_secret == client_secret:
            raise Exception(_("Invalid client_secret"))
    except Application.DoesNotExist:
        raise Exception(_("Invalid client_id"))

    vcode = VerificationCode.objects.filter(user=user).order_by("-created").first()
    if not vcode:
        raise Exception(_("User has not requested verification code"))

    if not vcode.validate(code, device_uuid=device_uuid):
        raise Exception(_("Invalid code for this user and device"))

    if vcode.device:
        vcode.device.last_login = timezone.now()
        vcode.device.save()

    # update code as verified
    vcode.verified = True
    vcode.save()

    if hasattr(user, "email_verified"):
        user.email_verified = True
    user.last_login = timezone.now()
    user.save()

    # Create access and refresh token
    expire_seconds = 60 * 60 * 24 * 30  # 30 days as default
    try:
        expire_seconds = settings.ACCESS_TOKEN_EXPIRE_SECONDS
    except:
        pass

    expires = timezone.now() + timezone.timedelta(seconds=expire_seconds)

    access_token = AccessToken.objects.create(
        user=user,
        expires=expires,
        scope="",
        token=common.generate_token(),
        application=oauth_app,
    )

    refresh_token = RefreshToken.objects.create(
        user=user,
        token=common.generate_token(),
        application=oauth_app,
        access_token=access_token,
    )
    return {
        "access_token": access_token.token,
        "refresh_token": refresh_token.token,
        "token_expire": access_token.expires,
    }


def get_or_create_project_role(project, role_name):
    """
    Create a role for project to provide multi project support
    Parameters:
    project (instance): Project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL
    role_name (str): Role name

    Returns:
    instance: nets_core.models.Role

    """
    try:
        project_model = apps.get_model(settings.NETS_CORE_PROJECT_MODEL)
        if not isinstance(project, project_model):
            raise Exception(
                _(
                    "Invalid project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL"
                )
            )

    except project_model.DoesNotExist:
        raise Exception(
            _(
                "Invalid project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL"
            )
        )

    content_type = ContentType.objects.get_for_model(project)
    if not role_name.endswith(f"_{project.id}"):
        role_name = f"{role_name}_{project.id}"
    role, _role_created = Role.objects.get_or_create(
        name=role_name, project_content_type=content_type, project_id=project.id
    )

    return role, _role_created


def get_or_create_project_role_permission(
    project, role_name, codename, verbose_name: str = None, description: str = ""
):
    """
    Create a role for project to provide multi project support
    Parameters:
    project (instance): Project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL
    role_name (str): Role name
    codename (str): Permission codename

    Returns:
    instance: nets_core.models.Role

    """
    role, _role_created = get_or_create_project_role(project, role_name)
    content_type = ContentType.objects.get_for_model(project)
    if not verbose_name:
        verbose_name = codename.replace("_", " ").capitalize()
    permission, _permission_created = Permission.objects.get_or_create(
        codename=codename, defaults={"name": verbose_name, "description": description}
    )

    role.permissions.add(permission)
    return permission, _permission_created


def add_user_to_role(user, project, role_name):
    """
    Add role to user in project
    Parameters:
    user (instance): User instance. Should be the same as settings.AUTH_USER_MODEL
    project (instance): Project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL
    role_name (str): Role name

    Returns:
    instance: nets_core.models.Role

    """
    try:
        project_model = apps.get_model(settings.NETS_CORE_PROJECT_MODEL)
        if not isinstance(project, project_model):
            raise Exception(
                _(
                    "Invalid project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL"
                )
            )

        user_model = apps.get_model(settings.AUTH_USER_MODEL)
        if not isinstance(user, user_model):
            raise Exception(
                _(
                    "Invalid user instance. Should be the same as settings.AUTH_USER_MODEL"
                )
            )

    except project_model.DoesNotExist:
        raise Exception(
            _(
                "Invalid project instance. Should be the same as settings.NETS_CORE_PROJECT_MODEL"
            )
        )

    role, _role_created = get_or_create_project_role(project, role_name)
    project_content_type = ContentType.objects.get_for_model(project)
    user_role, _user_role_created = UserRole.objects.get_or_create(
        user=user,
        role=role,
        project_content_type=project_content_type,
        project_id=project.id,
    )

    return user_role, _user_role_created


class SecureCache:
    """
    Secure cache to store and retrieve data with expiration time.
    This cache is secure because it uses the same key to store and retrieve data.
    It uses the same key to store and retrieve data.
    This cache
    Parameters:
    key (str): Key to store and retrieve data
    expiration (int): Time in seconds to expire data

    Returns:
    instance: SecureCache

    """

    def __init__(self):
        self.key = ""
        self.expiration = 0

    def secure_key(self, key: str) -> str:
        key_prefix = "NETS_SK_"
        # use settings.NETS_CORE_SECURE_CACHE_KEY or settings.SECRET_KEY as secret key
        secret_key = getattr(
            settings, "NETS_CORE_SECURE_CACHE_KEY", settings.SECRET_KEY
        )
        key = hmac.new(
            secret_key.encode("utf-8"), key.encode("utf-8"), hashlib.sha256
        ).hexdigest()

        k = f"{key_prefix}{key}"
        if len(k) > 250:
            k = k[:250]
        return k

    def secure_value(self, value: str) -> str: 
        # use settings.NETS_CORE_SECURE_CACHE_KEY or settings.SECRET_KEY as secret key
        secret_key = getattr(
            settings, "NETS_CORE_SECURE_CACHE_KEY", settings.SECRET_KEY
        )
        value = hmac.new(
            secret_key.encode("utf-8"), value.encode("utf-8"), hashlib.sha256
        ).hexdigest()
        return value

    def set(self, key: str, value: str, expiration: int) -> None:
        self.key = self.secure_key(key)
        self.expiration = expiration
        cache.set(self.key, self.secure_value(value), expiration)

    def get(self, key: str) -> str | None:
        """
        Get encrypted value from cache
        No decryption provided. Use secure_value to compare values
        Parameters:
        key (str): Key to get value

        Returns:
        str: Value or None if not found

        """
        self.key = self.secure_key(key)
        value = cache.get(self.key)
        if not value:
            return None
        return self.secure_value

    def delete(self, key: str) -> None:
        cache.delete(self.secure_key(key))
        self.key = ""
        self.expiration = 0

    def __str__(self) -> str:
        return self.key

    def __repr__(self) -> str:
        return self.key

    def __bool__(self) -> bool:
        return bool(self.key)

    def __len__(self) -> int:
        return len(self.key)

    def validate(self, key: str, value: str) -> bool:
        return self.secure_value(value) == self.get(key)



import os
import firebase_admin
from django.conf import settings
from firebase_admin import messaging
from firebase_admin.exceptions import FirebaseError
from firebase_admin import credentials
from nets_core.models import UserDevice, UserFirebaseNotification
import logging

logger = logging.getLogger(__name__)

firebase_config = os.getenv('FIREBASE_CONFIG')

if hasattr(settings, 'FIREBASE_CONFIG'):
        firebase_config = settings.FIREBASE_CONFIG

if not firebase_config:
    logger.warning('FIREBASE_CONFIG not set')
    # raise ValueError('FIREBASE_CONFIG not set')
else:
    try:
        cred = credentials.Certificate(firebase_config)
        default_app = firebase_admin.initialize_app(cred)
    except Exception as e:
        logger.error('Error initializing firebase', e)
    # raise ValueError('Error initializing firebase', e)


def send_fb_message(title:str, message:str, registration_token:str, data: dict=None, channel: str=None) -> str:

    try:
        android_config = messaging.AndroidConfig(
            priority='high',
            notification=messaging.AndroidNotification(
                icon='ic_launcher',
                color='#f45342',
                channel_id=channel or 'default'
            ),
        )
        message = messaging.Message(
            data=data,
            token=registration_token,
            notification=messaging.Notification(
                title=title,
                body=message
            ),
            android=android_config,
        )
        response = messaging.send(message)
        return response
    except Exception as e:
        
        raise ValueError('Error sending message', e)

    


def send_user_device_notification(user, title: str, message: str, data: dict=None, channel: str=None) -> dict:
    devices = UserDevice.objects.filter(user=user).exclude(last_login=None).exclude(firebase_token=None)
    devices_results = {}
    if devices:
        for device in devices:
            notification = UserFirebaseNotification.objects.create(
                    user=user, 
                    device=device, 
                    message=message, 
                    data=data, 
                )
            try:
                message_id = send_fb_message(title, message, device.firebase_token, data, channel)
                notification.message_id = message_id
                notification.sent = True
                notification.save()

                devices_results[device.id] = {
                    'success': True,
                    'message_id': message_id
                }
            except messaging.UnregisteredError as e:
                # delete device
                
                device.delete()

            except Exception as e:
                msg_error = str(e)
                if 'UnregisteredError' in msg_error:
                    device.delete()
                    continue
                logger.error(f'Error sending message {e}')
                message_id = None
                notification.error = str(e)
                notification.save()

                devices_results[device.id] = {
                    'success': False,
                    'error': str(e)
                }

    return devices_results
                
                


import json
import shortuuid

from uuid import uuid4


from django.conf import settings
from django.apps import apps
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.core.cache import cache
from django.contrib.auth.hashers import check_password, make_password
from django.utils import timezone
from django.contrib.auth import get_user_model


from nets_core.utils import generate_int_uuid

token_timeout_seconds = 15 * 60  # 15 minutes default
try:
    token_timeout_seconds = settings.NETS_CORE_VERIFICATION_CODE_EXPIRE_SECONDS
except Exception as e:
    # Settings not present use default
    pass


class NetsCoreBaseManager(models.Manager):

    def to_json(self, fields: tuple = None):
        query = self.get_queryset()
        if not query:
            raise ValueError(_("Query must be provided"))

        # get the query instance and check if JSON_DATA_FIELDS is present
        if hasattr(self.model, "JSON_DATA_FIELDS") and not fields:
            if not self.model.JSON_DATA_FIELDS:
                raise ValueError(_("Fields must be provided"))
            if not isinstance(self.model.JSON_DATA_FIELDS, tuple):
                try:
                    fields = tuple(self.model.JSON_DATA_FIELDS)
                except Exception as e:
                    raise ValueError(_("Fields must be a tuple or list"))

        if not fields:
            raise ValueError(_("Fields must be provided"))
        if fields == "__all__":
            # get fiels to tupple but names from columns in db table schema, user should be user_id
            schema = self.model._meta.get_fields()
            fields = tuple(
                [field.column for field in schema if hasattr(field, "column")]
            )

        if not isinstance(fields, tuple):
            raise ValueError(_("Fields must be a tuple"))

        from nets_core.serializers import NetsCoreModelToJson, NetsCoreQuerySetToJson

        if query.count() == 1:
            return NetsCoreModelToJson(query.first(), fields).to_json()

        return NetsCoreQuerySetToJson(query, fields).to_json()


class NetsCoreBaseModel(models.Model):
    created = models.DateTimeField(_("Created"), auto_now_add=True)
    updated = models.DateTimeField(_("updated"), auto_now=True)
    updated_fields = models.JSONField(
        _("Updated fields"), null=True, blank=True, default=dict
    )

    objects = NetsCoreBaseManager()

    class Meta:
        abstract = True

    def validate_fields(self, fields: tuple):
        schema = self._meta.get_fields()
        # add fields that are not in schema and end with _id as user should be user_id and not present in schema
        final_fields = [f for f in fields if f.endswith("_id")]
        for f in schema:
            if f.name in fields and not f.name in final_fields:
                if isinstance(f, models.ForeignKey):
                    related_model = f.related_model

                    if (
                        hasattr(related_model, "JSON_DATA_FIELDS")
                        and related_model.JSON_DATA_FIELDS
                    ):
                        if not isinstance(related_model.JSON_DATA_FIELDS, tuple):
                            try:
                                related_fields = tuple(related_model.JSON_DATA_FIELDS)
                            except Exception as e:
                                raise ValueError(_("Fields must be a tuple or list"))
                        else:
                            related_fields = fields
                        # replace field for related format as field_name:[table_name; ...related_fields]
                        table_name = related_model._meta.db_table
                        related_fields = [table_name] + list(related_fields)
                        final_fields.append(f"{f.name}_id:[{';'.join(related_fields)}]")
                        # final_fields = final_fields + (f"{f.name}:[{';'.join(related_fields)}]",)

                    else:
                        raise ValueError(
                            _(
                                "Field %s is a related model but has no JSON_DATA_FIELDS"
                                % f.name
                            )
                        )
                else:
                    final_fields.append(f.name)

        return tuple(final_fields)

    def to_json(self, fields: tuple = None):
        # check if JSON_DATA_FIELDS is present
        if hasattr(self, "JSON_DATA_FIELDS") and not fields:
            if not self.JSON_DATA_FIELDS:
                raise ValueError(_("Fields must be provided"))
            if not isinstance(self.JSON_DATA_FIELDS, tuple):
                try:
                    fields = tuple(self.JSON_DATA_FIELDS)
                except Exception as e:
                    raise ValueError(_("Fields must be a tuple or list"))

        if not fields:
            raise ValueError(_("Fields must be provided"))
        if fields == "__all__":
            # get fields to tuple but names from columns in db table schema, user should be user_id
            schema = self._meta.get_fields()
            fields = tuple(
                [field.column for field in schema if hasattr(field, "column")]
            )
            related_fields = [
                field.name for field in schema if hasattr(field, "related_model")
            ]
            fields = self.validate_fields(fields + tuple(related_fields))
        else:
            fields = self.validate_fields(fields)
        if not isinstance(fields, tuple):
            raise ValueError(_("Fields must be a tuple"))

        from nets_core.serializers import NetsCoreModelToJson

        return NetsCoreModelToJson(self, fields).to_json()

    def save(self, *args, **kwargs):

        super(NetsCoreBaseModel, self).save(*args, **kwargs)


class OwnedModel(NetsCoreBaseModel):
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)

    objects = NetsCoreBaseManager()

    class Meta:
        abstract = True


class Permission(NetsCoreBaseModel):
    name = models.CharField(_("Name"), max_length=150)
    codename = models.CharField(_("Codename"), max_length=150, unique=True)
    description = models.CharField(
        _("Description"), max_length=250, null=True, blank=True
    )
    # project_content_type = models.ForeignKey(
    #     ContentType,
    #     on_delete=models.CASCADE,
    #     null=True,
    #     blank=True,
    #     related_name="nets_core_permissions",
    # )
    # project_id = models.PositiveIntegerField(null=True, blank=True)
    # project = GenericForeignKey("project_content_type", "project_id")

    JSON_DATA_FIELDS = ["name", "codename", "description"]
    objects = NetsCoreBaseManager()

    class Meta:
        verbose_name = _("Permission")
        verbose_name_plural = _("Permissions")
        db_table = "nets_core_permission"

    def __str__(self):
        return f"{self.name} - {self.codename}"

    def save(self, *args, **kwargs):
        self.codename = self.codename.lower()
        if not self.name:
            self.name = self.codename

        super(Permission, self).save(*args, **kwargs)


class Role(NetsCoreBaseModel):
    name = models.CharField(_("Name"), max_length=150)
    codename = models.CharField(_("Codename"), max_length=150)
    description = models.CharField(_("Description"), max_length=250)
    permissions = models.ManyToManyField(
        Permission, related_name="roles", through="RolePermission"
    )
    project_content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, null=True, blank=True
    )
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey("project_content_type", "project_id")
    enabled = models.BooleanField(_("Enabled?"), default=True)

    JSON_DATA_FIELDS = [
        "name",
        "codename",
        "description",
        "project_id",
        "enabled",
    ]

    objects = NetsCoreBaseManager()

    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = _("Roles")
        db_table = "nets_core_role"
        indexes = [
            models.Index(
                fields=["project_content_type", "project_id"], name="role_index"
            )
        ]

    def __str__(self):
        if self.project:
            return f"{self.name} - {self.project}"
        return self.name

    def save(self, *args, **kwargs):
        self.codename = self.codename.lower()
        super(Role, self).save(*args, **kwargs)


class RolePermission(NetsCoreBaseModel):
    role = models.ForeignKey(Role, on_delete=models.CASCADE)
    permission = models.ForeignKey(Permission, on_delete=models.CASCADE)
    custom_name = models.CharField(
        _("Custom name"), max_length=150, null=True, blank=True
    )

    JSON_DATA_FIELDS = ["role_id", "permission_id", "custom_name"]
    objects = NetsCoreBaseManager()

    class Meta:
        verbose_name = _("Role Permission")
        verbose_name_plural = _("Role Permissions")
        db_table = "nets_core_role_permission"

    def __str__(self):
        if self.project:
            return f"{self.role} - {self.permission} - {self.project}"
        return f"{self.role} - {self.permission}"

    def save(self, *args, **kwargs):
        if hasattr(self, "project") and self.project:
            self.project_content_type = ContentType.objects.get_for_model(self.project)
            self.project_id = self.project.id

        super(RolePermission, self).save(*args, **kwargs)


class UserRole(NetsCoreBaseModel):
    user = models.ForeignKey(
        get_user_model(), on_delete=models.CASCADE, related_name="roles"
    )
    role = models.ForeignKey(Role, on_delete=models.CASCADE)
    project_content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, null=True, blank=True
    )
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey("project_content_type", "project_id")

    objects = NetsCoreBaseManager()

    JSON_DATA_FIELDS = ["user_id", "role_id", "project_content_type", "project_id"]

    class Meta:
        verbose_name = _("User Role")
        verbose_name_plural = _("User Roles")
        db_table = "nets_core_user_role"

    def __str__(self):
        if self.project:
            return f"{self.user} - {self.role} - {self.project}"
        return f"{self.user} - {self.role}"

    def save(self, *args, **kwargs):
        if hasattr(self, "project") and self.project:
            self.project_content_type = ContentType.objects.get_for_model(self.project)
            self.project_id = self.project.id

        super(UserRole, self).save(*args, **kwargs)


class VerificationCode(OwnedModel):
    token = models.CharField(_("Verification token"), max_length=150)
    device = models.ForeignKey(
        "nets_core.UserDevice",
        max_length=150,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    verified = models.BooleanField(_("Verified?"), default=False)
    ip = models.CharField(_("IP"), max_length=150, null=True, blank=True)

    class Meta:
        db_table = "nets_core_verification_code"

    def get_token_cache_key(self):
        token_key_prefix = "NC_T"
        try:
            token_key_prefix = settings.NETS_CORE_VERIFICATION_CODE_CACHE_KEY
        except:
            pass

        return f"{token_key_prefix}{self.user.pk}"

    def save(self, *args, **kwargs):
        token = 123456
        cache_token_key = self.get_token_cache_key()
        tester_emails = ["google_tester*"]
        # check if settings has testers emails
        if (
            hasattr(settings, "NETS_CORE_TESTERS_EMAILS")
            and type(settings.TESTERS_EMAILS) == list
        ):
            tester_emails += settings.NETS_CORE_TESTERS_EMAILS

        is_tester = False
        for tester_email in tester_emails:
            # emails can be like google_tester* or google_tester1@gmail.com
            if tester_email.endswith("*"):
                if self.user.email.startswith(tester_email.replace("*", "")):
                    is_tester = True
                    break
            else:
                if self.user.email == tester_email:
                    is_tester = True
                    break

        if is_tester:
            token = 789654
            if hasattr(settings, "NETS_CORE_TESTERS_VERIFICATION_CODE"):
                token = settings.NETS_CORE_TESTERS_VERIFICATION_CODE

        if not settings.DEBUG and not is_tester:
            # Check cache if token is present and return the same token
            token = cache.get(cache_token_key)
            if not token:
                # Generate a new numeric token six digits
                token = generate_int_uuid(6)

        # Set token in cache system
        cache.set(cache_token_key, "{}".format(token), token_timeout_seconds)

        # Encrypt the token and send email
        self.token = make_password(str(token))
        super(VerificationCode, self).save(*args, **kwargs)

    def validate(self, token: str = None, device_uuid: str = None):
        if not token or not self.token:
            return False

        if self.device:
            if not device_uuid or self.device.uuid != device_uuid:
                return False

        if (timezone.now() - self.created).total_seconds() > token_timeout_seconds:
            # code expired delete it.
            self.delete()
            return False

        return check_password(token, self.token)


TEMPLATES_USES = (
    ("verification_code", _("Verification code: send each time a code is generated")),
    (
        "other",
        _(
            "Other: Nets core not implemented use. You can use this from any view quering from nets_core.models.EmailTemplate"
        ),
    ),
)


class EmailTemplate(OwnedModel):
    name = models.CharField(_("Name"), max_length=150, unique=True)
    html_body = models.TextField(
        _("HTML BODY"),
        help_text=_(
            "You can use Django template language see: https://docs.djangoproject.com/en/4.1/ref/templates/language/ "
        ),
    )
    text_body = models.TextField(
        _("TXT BODY"),
        help_text=_(
            "You can use Django template language see: https://docs.djangoproject.com/en/4.1/ref/templates/language/ "
        ),
    )
    enabled = models.BooleanField(_("Enabled?"), default=True)
    use_for = models.CharField(
        _("Use template for"), default="other", choices=TEMPLATES_USES, max_length=50
    )
    project_content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, null=True, blank=True
    )
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey("project_content_type", "project_id")
    JSON_DATA_FIELDS = [
        "name",
        "html_body",
        "text_body",
        "enabled",
        "use_for",
        "project_content_type",
        "project_id",
    ]

    class Meta:
        verbose_name = _("Email Template")
        verbose_name_plural = _("Email Templates")
        db_table = "nets_core_email_template"
        indexes = [
            models.Index(
                fields=["project_content_type", "project_id"],
                name="email_template_index",
            )
        ]

    def __str__(self) -> str:
        if self.project:
            return f"{self.name} - {self.project}"
        return self.name


class CustomEmail(OwnedModel):
    subject = models.CharField(_("subject"), max_length=250)
    to_email = models.TextField(_("To Email"), max_length=250)
    from_email = models.CharField(_("From Email"), max_length=250)
    html_body = models.TextField(_("HTML Body"))
    txt_body = models.TextField(_("TXT Body"), null=True, blank=True)
    completed = models.BooleanField(_("completed"), default=False)
    sent_count = models.IntegerField(_("Sent count"), default=0)
    failed_count = models.IntegerField(_("Failed count"), default=0)
    project_content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, null=True, blank=True
    )
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey("project_content_type", "project_id")
    JSON_DATA_FIELDS = [
        "subject",
        "to_email",
        "from_email",
        "html_body",
        "txt_body",
        "completed",
        "sent_count",
        "failed_count",
    ]

    class Meta:
        verbose_name = _("Email")
        verbose_name_plural = _("Emails")
        db_table = "nets_core_custom_email"
        indexes = [
            models.Index(
                fields=["project_content_type", "project_id"], name="custom_email_index"
            )
        ]

    def __str__(self):
        return self.subject


class EmailNotification(NetsCoreBaseModel):
    subject = models.CharField(_("subject"), max_length=250)
    to = models.TextField(_("To Email"), max_length=250)
    from_email = models.CharField(_("From Email"), max_length=250)
    body = models.TextField(_("HTML Body"))
    txt_body = models.TextField(_("TXT Body"), null=True, blank=True)
    sent = models.BooleanField(_("sent"), default=False)
    tries = models.IntegerField(_("tries"), default=0)
    sent_at = models.DateTimeField(_("Sent at"), null=True)
    created = models.DateTimeField(_("created"), auto_now_add=True)
    updated = models.DateTimeField(_("created"), auto_now=True)
    custom_email = models.ForeignKey(CustomEmail, null=True, on_delete=models.CASCADE)
    project_content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, null=True, blank=True
    )
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey("project_content_type", "project_id")
    JSON_DATA_FIELDS = [
        "subject",
        "to",
        "from_email",
        "body",
        "txt_body",
        "sent",
        "tries",
        "sent_at",
        "created",
        "updated",
        "custom_email_id",
    ]

    def __str__(self):
        if self.project:
            return "%s %s %s" % (self.to, self.subject, self.project)

        return "%s %s" % (self.to, self.subject)

    class Meta:
        verbose_name = _("Email Notification")
        verbose_name_plural = _("Email Notifications")
        db_table = "nets_core_email_notification"
        indexes = [
            models.Index(
                fields=["project_content_type", "project_id"],
                name="email_notification_index",
            )
        ]


class UserDevice(OwnedModel):
    uuid = models.UUIDField(_("UUID"), default=uuid4, editable=False, unique=True)
    name = models.CharField(_("Name"), max_length=250)
    os = models.CharField(_("OS"), max_length=250, null=True, blank=True)
    os_version = models.CharField(
        _("OS Version"), max_length=250, null=True, blank=True
    )
    app_version = models.CharField(
        _("App Version"), max_length=250, null=True, blank=True
    )
    device_type = models.CharField(
        _("Device Type"), max_length=250, null=True, blank=True
    )
    device_token = models.CharField(
        _("Device Token"), max_length=250, null=True, blank=True
    )
    firebase_token = models.CharField(
        _("Firebase Token"), max_length=250, null=True, blank=True
    )
    active = models.BooleanField(_("Active"), default=True)
    last_login = models.DateTimeField(_("Last login"), null=True)
    ip = models.CharField(_("IP"), max_length=250, null=True, blank=True)
    JSON_DATA_FIELDS = [
        "uuid",
        "user_id",
        "name",
        "os",
        "os_version",
        "app_version",
        "device_type",
        "active",
        "last_login",
        "ip",
    ]
    PROTECTED_FIELDS = ["device_token", "firebase_token"]

    class Meta:
        verbose_name = _("User Device")
        verbose_name_plural = _("User Devices")
        db_table = "nets_core_user_device"

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):

        super(UserDevice, self).save(*args, **kwargs)


class UserFirebaseNotification(OwnedModel):
    message = models.TextField(_("Message"), null=True, blank=True)
    devices = models.ManyToManyField(UserDevice, related_name="notifications")
    data = models.JSONField(_("Data"), null=True, blank=True)
    sent = models.BooleanField(_("Sent"), default=False)
    message_id = models.CharField(
        _("Message ID"), max_length=250, null=True, blank=True
    )
    error = models.TextField(_("Error"), null=True, blank=True)
    device = models.ForeignKey(UserDevice, on_delete=models.CASCADE)

    JSON_DATA_FIELDS = [
        "message",
        "data",
        "sent",
        "message_id",
        "error",
        "device_id",
        "user_id",
    ]
    
    class Meta:
        verbose_name = _("User Firebase Notification")
        verbose_name_plural = _("User Firebase Notifications")
        db_table = "nets_core_firebase_notification"

    def __str__(self):
        return self.message_id



ASGI_APPLICATION = 'nets_core.asgi.application'
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.memcached.PyMemcacheCache",
        "LOCATION": "127.0.0.1:11211"
    }
}
CELERY_ACCEPT_CONTENT = [
    'application/json',
]
CELERY_BROKER_URL = 'redis://localhost:6379/0'
CELERY_RESULT_BACKEND = 'redis://localhost:6379/0'
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TASK_SERIALIZER = 'json'
CELERY_TIMEZONE = 'UTC'
CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.core.RedisChannelLayer",
        "CONFIG": {
            "hosts": [
                [
                    "127.0.0.1",
                    6379
                ]
            ],
            "capacity": 1500,
            "expiry": 10
        }
    },
    "dev": {
        "BACKEND": "channels.layers.InMemoryChannelLayer"
    }
}
ENV = 'DEV'
INSTALLED_APPS = [
    'daphne',
    'channels',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'oauth2_provider',
    'nets_core',
    'django_celery_beat',
    'corsheaders',
]
MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'oauth2_provider.middleware.OAuth2TokenMiddleware',
    'nets_core.middleware.auth_token.AuthTokenMiddleware',
]
ROOT_URLCONF = 'nets_core.auth_urls'
SITE_DOMAIN = 'test'
SITE_NAME = 'test'



"""
ASGI config for nets_core project.

It exposes the ASGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/howto/deployment/asgi/
"""

import os

from django.core.asgi import get_asgi_application
from nets_core.middleware.auth_token import AuthTokenMiddleware
from channels.routing import ProtocolTypeRouter, URLRouter

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'nets_core.settings')

django_asgi_application = get_asgi_application()

application = ProtocolTypeRouter({
    "http": django_asgi_application,
    #"websocket": AuthTokenMiddleware(
        # URLRouter(
            # Just like the Django URL router, but for WebSockets
            # "nets_core.routing.websocket_urlpatterns"
    #   )
    # ),
})



import json
from django.db import models, connections
from django.conf import settings
from django.utils.translation import gettext_lazy as _

GLOBAL_PROTECTED_FIELDS = [
    'password',
    'is_active',
    'enabled',
    'staff',
    'superuser',
    'verified',
    'deleted',
    'token',
    'auth',
    'perms',
    'groups',
    'ip'
    'date_joined',
    'last_login',
    
]

if hasattr(settings, "NETS_CORE_GLOBAL_PROTECTED_FIELDS"):
    GLOBAL_PROTECTED_FIELDS = settings.NETS_CORE_GLOBAL_PROTECTED_FIELDS

class NetsCoreQuerySetToJson():
    
    def __init__(self, queryset: models.QuerySet, fields: tuple = None, using: str = "default"):
        if not fields:
            # check if instance has JSON_FIELDS attribute
            if hasattr(queryset.model, "JSON_DATA_FIELDS"):
                if not queryset.model.JSON_DATA_FIELDS:
                    raise ValueError(_("Fields must be provided"))
                if not isinstance(queryset.model.JSON_DATA_FIELDS, tuple):
                    try:
                        fields = tuple(queryset.model.JSON_DATA_FIELDS)
                    except Exception as e:
                        raise ValueError(_("Fields must be a tuple or list"))
            else:
                
                raise ValueError(_("Fields must be provided"))
        
        if not isinstance(fields, tuple):
            raise ValueError(_("Fields must be a tuple"))
        
        if not isinstance(queryset, models.QuerySet):
            raise ValueError(_("Queryset must be a queryset instance or subclass of models.QuerySet"))
        
        if not isinstance(using, str):
            raise ValueError(_("Using must be a string"))
        
        if not using in connections.databases:
            raise ValueError(_("Database alias not found"))
        
        self.queryset = queryset
        if len(queryset) > 1:
            instance = queryset.first()
            # check if instance has protected_fields
            if hasattr(instance, "PROTECTED_FIELDS"):
                # get the protected fields
                protected_fields = [f.lower() for f in instance.PROTECTED_FIELDS]
                # remove the protected fields from the fields
                self.fields = [field for field in fields if field.lower() not in protected_fields]
                
            else:
                # remove the global protected fields from the fields using contains for each GLOBAL_PROTECTED_FIELDS
                for field in GLOBAL_PROTECTED_FIELDS:
                    for i, f in enumerate(fields):
                        if field.lower() in f.lower():
                            fields = fields[:i] + fields[i+1:]
        self.fields = ", ".join(fields)
        self.using = using
        
    def to_json(self):
        
        query_ids = ", ".join([str(obj.pk) for obj in self.queryset])
        with models.connections[self.using].cursor() as cursor:
            cursor.execute(f"SELECT nets_core_postgre_array_model_to_json('{self.queryset.model._meta.db_table}', '{self.fields}', ARRAY[{query_ids}])")
            row = cursor.fetchone()
            return row[0]
    

class NetsCoreModelToJson():

    def __init__(self, instance: models.Model, fields: tuple = None, using: str = "default"):
        if not fields:
            # check if instance has JSON_FIELDS attribute
            if hasattr(instance, "JSON_DATA_FIELDS"):
                if not instance.JSON_DATA_FIELDS:
                    raise ValueError(_("Fields must be provided"))
                if not isinstance(instance.JSON_DATA_FIELDS, tuple):
                    try:
                        fields = tuple(instance.JSON_DATA_FIELDS)
                    except Exception as e:
                        raise ValueError(_("Fields must be a tuple or list"))
            else:
                raise ValueError(_("Fields must be provided"))            
            
        
        if not isinstance(fields, tuple):
            raise ValueError(_("Fields must be a tuple"))
        
        if not isinstance(instance, models.Model):
            raise ValueError(_("Instance must be a model instance or subclass of models.Model"))
        
        if not isinstance(using, str):
            raise ValueError(_("Using must be a string"))
        
        if not using in connections.databases:
            raise ValueError(_("Database alias not found"))
        
        self.instance = instance

        # check if instance has protected_fields
        if hasattr(instance, "PROTECTED_FIELDS"):
            # get the protected fields
            protected_fields = [f.lower() for f in instance.PROTECTED_FIELDS]
            # remove the protected fields from the fields
            self.fields = [field for field in fields if field.lower() not in protected_fields]
            
        else:
            # remove the global protected fields from the fields
            for field in GLOBAL_PROTECTED_FIELDS:
                for i, f in enumerate(fields):
                    if field.lower() in f.lower():
                        fields = fields[:i] + fields[i+1:]

        self.fields = ",".join(fields)
        self.using = using
        
    def to_json(self, returning_query: bool = False):
        with connections[self.using].cursor() as cursor:
            t_query = f"SELECT nets_core_postgre_model_to_json('{self.instance._meta.db_table}', '{self.fields}', {self.instance.pk})"
            if returning_query:
                return t_query
            cursor.execute(t_query)
            row = cursor.fetchone()
            if not row:
                return None
            return row[0]
        



NETS_CORE_PROTECTED_FIELDS = [
    'password',
    'is_active',
    'enabled',
    'staff',
    'superuser',
    'verified',
    'deleted',
    'token',
    'auth',
    'perms',
    'groups',
    'ip',
    'email',
    'doc',
    'permissions',
    'date_joined',
    'last_login',
    'verified',
    'updated_fields'
]


# Generated by Django 5.0.7 on 2024-07-26 19:23

import django.db.models.deletion
from django.db import migrations, models


def pre_migration(apps, schema_editor):
    Role = apps.get_model("nets_core", "Role")
    Permission = apps.get_model("nets_core", "Permission")

    ContentType = apps.get_model("contenttypes", "ContentType")
    unique_codenames = []
    for p in Permission.objects.all():
        if p.codename not in unique_codenames:
            unique_codenames.append(p.codename)
            other_permissions = Permission.objects.filter(codename=p.codename).exclude(
                id=p.id
            )
            if other_permissions.count() == 0:
                # update role permissions with the new permission
                for op in other_permissions:
                    for r in op.roles.all():
                        r.permissions.add(p)
                        r.permissions.remove(op)
                        r.save()
                # delete other permissions
                other_permissions.delete()
        else:
            p.delete()


def migrate_through(apps, schema_editor):
    Role = apps.get_model("nets_core", "Role")
    Permission = apps.get_model("nets_core", "Permission")
    RolePermission = apps.get_model("nets_core", "RolePermission")

    for role in Role.objects.all():
        for permission in role.permissions.all():
            RolePermission.objects.create(role=role, permission=permission)


class Migration(migrations.Migration):

    dependencies = [
        ("nets_core", "pre_migrate_role_permissions"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="permission",
            name="permission_index",
        ),
        migrations.RemoveField(
            model_name="permission",
            name="project_content_type",
        ),
        migrations.RemoveField(
            model_name="permission",
            name="project_id",
        ),
        migrations.AlterField(
            model_name="permission",
            name="codename",
            field=models.CharField(
                max_length=150, unique=True, verbose_name="Codename"
            ),
        ),
        migrations.RemoveField(
            model_name="role",
            name="permissions",
        ),
        migrations.AddField(
            model_name="role",
            name="permissions",
            field=models.ManyToManyField(
                related_name="roles",
                through="nets_core.RolePermission",
                to="nets_core.permission",
            ),
        ),
    ]



from nets_core import firebase_messages
from django.conf import settings
from django.core.management.base import BaseCommand
from django.apps import apps

import logging

logger = logging.getLogger(__name__)

    
class Command(BaseCommand):
    help = 'Send push notification to user_id or firebase_token'
    
    def add_arguments(self, parser):
        parser.add_argument('--user_id', nargs='?', type=int)
        parser.add_argument('--firebase_token', nargs='?', type=str)
        parser.add_argument('--title', nargs='?', type=str)
        parser.add_argument('--message', nargs='?', type=str)

    def handle(self, *args, **options):
        title = options['title'] or 'Planyz'
        message = options['message'] or 'Hello from Planyz (Using command test_push_notification)'
        
        # get user
        if options['user_id']:            
            self.user_notification(options['user_id'], title, message)
            
        elif options['firebase_token']:
            try:
                result = firebase_messages.send_fb_message(
                    title,
                    message,
                    options['firebase_token']
                )
                logger.info(self.style.SUCCESS(f'Message sent: {result}'))
            except Exception as e:
                logger.error(self.style.ERROR(f'Error sending message: {e}'))
        else:
            print('user_id or firebase_token is required')
            exit(1)
            
            
    def user_notification(self, user_id: int, title: str, message: str):
        try:
            User = apps.get_model(settings.AUTH_USER_MODEL)
        except Exception as e:
            logger.error(self.style.ERROR(f'Error getting user model: {e}'))
            exit(1)
        
        user = User.objects.filter(id=user_id).first()
        if not user:
            logger.error(self.style.ERROR(f'User not found'))
            exit(1)
            
        try:
            for device in user.userdevice_set.all():
                
                print(f'Sending message to {device.firebase_token}')
                result = firebase_messages.send_fb_message(
                    title,
                    message,
                    device.firebase_token
                )
                logger.info(self.style.SUCCESS(f'Message sent: {result}'))
        except Exception as e:
            logger.error(self.style.ERROR(f'Error sending message: {e}'))
            exit(1)


import os

from celery import Celery

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'nets_core.settings')

app = Celery('nets_core')

app.config_from_object('django.conf:settings', namespace='CELERY')

app.autodiscover_tasks()



# Generated by Django 5.0.7 on 2024-07-26 19:23

import django.db.models.deletion
from django.db import migrations, models


def pre_migration(apps, schema_editor):
    Role = apps.get_model("nets_core", "Role")
    Permission = apps.get_model("nets_core", "Permission")

    ContentType = apps.get_model("contenttypes", "ContentType")
    unique_codenames = []
    for p in Permission.objects.all():
        if p.codename not in unique_codenames:
            unique_codenames.append(p.codename)
            other_permissions = Permission.objects.filter(codename=p.codename).exclude(
                id=p.id
            )
            if other_permissions.count() == 0:
                # update role permissions with the new permission
                for op in other_permissions:
                    for r in op.roles.all():
                        r.permissions.add(p)
                        r.permissions.remove(op)
                        r.save()
                # delete other permissions
                other_permissions.delete()
        else:
            p.delete()


def migrate_through(apps, schema_editor):
    Role = apps.get_model("nets_core", "Role")
    Permission = apps.get_model("nets_core", "Permission")
    RolePermission = apps.get_model("nets_core", "RolePermission")

    for role in Role.objects.all():
        for permission in role.permissions.all():
            RolePermission.objects.create(role=role, permission=permission)


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("nets_core", "0013_customemail_updated_fields_and_more"),
        
    ]

    operations = [
        migrations.RunPython(pre_migration),
        migrations.CreateModel(
            name="RolePermission",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="updated"),
                ),
                (
                    "updated_fields",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        null=True,
                        verbose_name="Updated fields",
                    ),
                ),
                (
                    "custom_name",
                    models.CharField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="Custom name",
                    ),
                ),
            ],
            options={
                "verbose_name": "Role Permission",
                "verbose_name_plural": "Role Permissions",
                "db_table": "nets_core_role_permission",
            },
        ),
        migrations.AddField(
            model_name="rolepermission",
            name="role",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="nets_core.role"
            ),
        ),
        migrations.AddField(
            model_name="rolepermission",
            name="permission",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="nets_core.permission",
            ),
        ),                
        migrations.RunPython(migrate_through),
    ]
